//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function M = inertia(robot, q)
// Manipulator inertia matrix
//
// Calling Sequence
// I = inertia(R,Q)
//
// Description
// I = inertia(R,Q) is the symmetric joint inertia matrix (NxN) which relates 
// joint torque to joint acceleration for the robot at joint configuration Q.
//
// If Q is a matrix (KxN), each row is interpretted as a joint state 
// vector, and the result is a 3d-matrix (NxNxK) where each plane corresponds
// to the inertia for the corresponding row of Q.
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>The diagonal elements I(J,J) are the inertia seen by joint actuator J.</para></listitem>
// <listitem><para>The off-diagonal elements I(J,K) are coupling inertias that relate
// acceleration on joint J to force/torque on joint K.</para></listitem>
// <listitem><para>The diagonal terms include the motor inertia reflected 
// through the gear ratio.</para></listitem>
// </itemizedlist>
// </refsection>
//
// See also
// rne
// cinertia 
// itorque
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    if numcols(q) ~= robot.n
        error(msprintf(gettext('q must have %d columns'), robot.n));
    end

    if numrows(q) > 1
        M = [];
        for i=1:numrows(q)
            M = cat(3, M, inertia(robot,q(i,:)));
        end
        return
    end

	n = robot.n;

	if length(q) == robot.n
		q = q(:).';
	end

	M = zeros(n,n,0);
	for i = 1:size(q,1),
		M(:,:,i) = rne(robot, ones(n,1)*q(i,:), zeros(n,n), eye(n,n), [0;0;0]);
	end
endfunction
