//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function T = linkTransf(L, q)
    // Link transform matrix
    //
    // Calling Sequence 
    // T = linkTransf(L,Q)  
    //
    // Parameters
    // L    : Link object
    // Q	: vector of size (1xN) where N is the number of joints of the robot.
    //
    // Description
    // <literal>T = linkTransf(L,Q)</literal> is the link homogeneous transformation matrix (4x4) corresponding
    // to the link variable Q which is either the Denavit-Hartenberg parameter THETA (revolute)
    // or D (prismatic).
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>For a revolute joint the THETA parameter of the link is ignored, and Q used instead.</para></listitem>
    // <listitem><para>For a prismatic joint the D parameter of the link is ignored, and Q used instead.</para></listitem>
    // <listitem><para>The link offset parameter is added to Q before computation of the transformation matrix.</para></listitem>
    // </itemizedlist>   
    // </refsection>
    //
    // See also 
    //	Link
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    sa = sin(L.alpha); ca = cos(L.alpha);
    q = q + L.offset;
    if L.sigma == 0
        // revolute
        st = sin(q); 
        ct = cos(q);
        d = L.d;
    else
        // prismatic
        st = sin(L.theta); 
        ct = cos(L.theta);
        d = q;
    end

    if L.mdh == 0
        // standard DH
        T =[    ct  -st*ca  st*sa   L.a*ct
        st  ct*ca   -ct*sa  L.a*st
        0   sa      ca      d
        0   0       0       1];
    else
        // modified DH
        T =[    ct      -st     0   L.a
        st*ca   ct*ca   -sa -sa*d
        st*sa   ct*sa   ca  ca*d
        0       0       0   1];
    end

endfunction
