//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [rnew] = plot_robot(robot, tg, varargin)

    // Graphical display and animation
    //
    // Calling Sequence
    // plot_robot(R,Q, OPTIONS)
    //
    // Description
    // plot_robot(R,Q, options) displays a graphical animation of a robot based on 
    // the kinematic model.  A stick figure polyline joins the origins of
    // the link coordinate frames. The robot is displayed at the joint angle Q (1xN), or 
    // if a matrix (MxN) it is animated as the robot moves along the M-point trajectory.
    //
    // <refsection>
    //   <title>Options</title>
    //   <variablelist>
    //   <varlistentry><term>"workspace", W :</term>
    //         <listitem><para>size of robot 3D workspace, W = [xmn, xmx ymn ymx zmn zmx]</para></listitem></varlistentry>
    //   <varlistentry><term>"delay", d :</term>
    //         <listitem><para>delay betwen frames for animation (s)</para></listitem></varlistentry>
    //   <varlistentry><term>"fps",fps   :</term>
    //         <listitem><para>set number of frames per second for display</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]loop"  :</term>
    //         <listitem><para>loop over the trajectory forever</para></listitem></varlistentry>
    //   <varlistentry><term>"mag", scale :</term>
    //         <listitem><para>annotation scale factor</para></listitem></varlistentry>
    //   <varlistentry><term>"cylinder", C :</term>
    //         <listitem><para>color for joint cylinders, C=[r g b]</para></listitem></varlistentry>
    //   <varlistentry><term>"ortho" :</term>
    //         <listitem><para> orthogonal camera view (default)</para></listitem></varlistentry>
    //   <varlistentry><term>"perspective"  :</term>
    //         <listitem><para>perspective camera view</para></listitem></varlistentry>
    //   <varlistentry><term>"xyz" :</term>
    //         <listitem><para> wrist axis label is XYZ</para></listitem></varlistentry>
    //   <varlistentry><term>"noa"   :</term>
    //         <listitem><para>wrist axis label is NOA</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]raise"  :</term>
    //         <listitem><para>autoraise the figure (very slow)</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]render"   :</term>
    //         <listitem><para>controls shaded rendering after drawing</para></listitem></varlistentry>
    //   <varlistentry><term> "[no]base" :</term>
    //         <listitem><para>controls display of base "pedestal"</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]wrist"  :</term>
    //         <listitem><para>controls display of wrist</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]shadow" :</term>
    //         <listitem><para>controls display of shadow</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]name"  :</term>
    //         <listitem><para>display the robot"s name</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]jaxes" :</term>
    //         <listitem><para> control display of joint axes</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]hold" :</term>
    //         <listitem><para> draw a robot in the last robot figure</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]joints"   :</term>
    //         <listitem><para>controls display of joints</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    //
    // The options come from 3 sources and are processed in order:
    //
    // - Cell array of options returned by the function PLOTBOTOPT (if it exists)
    //
    // - Cell array of options given by the 'plotopt' option when creating the
    //   SerialLink object.
    //
    // - List of arguments in the command line.
    //
    // Many boolean options can be enabled or disabled with the 'no' prefix.  The
    // various option sources can toggle an option, the last value is taken.
    //
    // To save the effort of processing options on each call they can be preprocessed by
    //        opts = robot.plot({'opt1', 'opt2', ... });
    // and the resulting object can be passed in to subsequent calls instead of text-based
    // options, for example:
    //        robot.plot(q, opts);
    //
    // Graphical annotations and options:
    //
    // The robot is displayed as a basic stick figure robot with annotations 
    // such as: shadow on the floor, XYZ wrist axes and labels, 
    // joint cylinders and axes, which are controlled by options.
    //
    // The size of the annotations is determined using a simple heuristic from 
    // the workspace dimensions.  This dimension can be changed by setting the 
    // multiplicative scale factor using the 'mag' option.
    //
    // Figure behaviour:
    //
    // - If no figure exists one will be created and teh robot drawn in it.
    //
    // - If no robot of this name is currently displayed then a robot will
    //   be drawn in the current figure.  If hold is enabled (hold on) then the
    //   robot will be added to the current figure.
    //
    // - If the robot already exists then that graphical model will be found 
    //   and moved.
    //
    // Multiple views of the same robot:
    //
    // If one or more plots of this robot already exist then these will all
    // be moved according to the argument Q.  All robots in all windows with 
    // the same name will be moved.
    //
    // Examples
    //  //Create a robot in figure 1
    //         mdl_puma560;
    //         handle_1 = scf(1)
    //         handle_1.tag = "Robot"
    //         plot_robot(p560,qz);
    //  //Create a robot in figure 2
    //         handle_2 = scf(2);
    //         handle_2.tag = "Robot"
    //         plot_robot(p560,qz);
    //  //Now move both robots
    //         plot_robot(p560,qn);
    //
    // Examples
    // //Multiple robots in the same figure:
    //
    //  //Create a robot in figure 1
    //         mdl_puma560;
    //         plot_robot(p560,qz);
    //  //Make a clone of the robot named bob
    //         bob = SerialLink(p560, 'name', 'bob');
    //         bob.base = transl(0, 0.5, -0.5);
    //  //Draw bob in this figure
    //         plot_robot(bob,qn,'hold')
    //
    //  //To animate both robots so they move together:
    //         qtg = jtraj(qr, qz, 100);
    //         for q=qtg'
    //           plot_robot(p560,q');
    //           plot_robot(bob,q');
    //         end
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>Delay betwen frames can be eliminated by setting option 'delay', 0 or
    // 'fps', 0</para></listitem>
    // <listitem><para>By default a quite detailed plot is generated, but turning off labels,
    // axes, shadows etc. will speed things up.</para></listitem>
    // <listitem><para>The graphical state holds the last joint configuration which 
    // can be retrieved using q = robot.plot().</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // SerialLink
    // plot_options
    // animate 
    // fkine
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    [%nargout,%nargin] = argn(0);
    rnew = [];
    // q = PLOT(robot)
    // return joint coordinates from a graphical robot of given name
    if isempty(tg) then                    // if nargin is 1

        rh = rt_findtag(robot.name);
        if ~isempty(rh) then
            userdata = rh(1).user_data;
            r = userdata(2);                    // robot's info are in user_data(2)
            rnew = r.q;
        end
        return

        // robot = PLOT(robot, q)
    else
        // based on standard or modified DH notation?
        rt_animate = "rt_animate_std";
        if robot.mdh then
            rt_animate = "rt_animate_mod";
        end
    end

    // robot2 = PLOT(robot, q, options)
    if %nargin > 2 then
        options = varargin
    else
        options = list();
    end

    opt = plot_options(robot,options);
    old_driver = driver();
    driver("X11");
    rh_computed = %F;
    rh = [];
    point_of_view = [35, 45];
    np = size(tg,1);
    n = robot.n;
    //    opt.repeat = 1; per farlo andare devo usare getfield
    opt.loop = 0;
    if size(tg,2) ~= n then
        error("insufficient columns in q")
    end


    // else do the right thing with figure windows.
    // does already exists any figure?
    fh = winsid();
     
//     close the demo window if it exists
//     (it could cause the spawn of infinite white windows)   
//    for i=1:length(fh)
//        cfh = get_figure_handle(fh(i));
//        if cfh.tag == "scilab_demo_fig" then
//            warning('Demo figure opened. It can cause some problems with the Robotics Toolbox');
//            close(cfh);
//            fh = winsid();
//        end
//    end
    
    if isempty(fh)
        
        cfh = scf();
//        disp(cfh.figure_id,"1) no figures exist at all, create one. ID = ");
        drawlater();
        
        // process options
        opt = plot_options(robot,options);
        h = create_new_robot(robot,opt);
        cfh.tag = "Robot";
        // save the handles in the passed robot object, and
        // attach it to the robot as user data.
        robot.handles = h;
        // robot name in user_data(1)
        // robot info in user_data(2)
        h.links.user_data = list(robot.name, SerialLink(robot));
    else
        // at least a figure exist
        cfh = gcf();
//        disp(cfh.figure_id, "1) at least a figure exist cfh ID = ")
        //--------------------------------------------------------//
        //    check if figure's tag is 'robot'                    //
        //--------------------------------------------------------//
        if  cfh.tag ~= "Robot" then
            // 
            // if tag isn't Robot search an existing figure with that tag
            // if it doesn't exists create a new one
            figures = winsid();
            i = length(figures);
            robotfigure = %f;
            while (robotfigure == %f & i > 0)
                cfh = get_figure_handle(figures(i));
                if cfh.tag == "Robot" then
                  disp("robot figure found")
                    cfh = scf(figures(i));
                    robotfigure = %t; //exit whit this figure
                end
                i = i-1;
            end
            
            if  robotfigure == %f & length(figures) < 20 then
                // there isn't figure with tag == Robot
                // find the next free ID starting from the smallest existing ID
                // (avoid to create white windows if there are unused IDs. 
                //  useful with cmscope during Xcos simulations). 

                nextID = min(figures)+1;
                while ~isempty(find(figures==nextID))
                    nextID = nextID + 1;
                end
                cfh = scf(nextID);
                cfh.children.view ="3d";
                cfh.tag = "Robot";
//                disp(cfh.figure_id," 1.5) robot figure not found, create new figure ID = ");
            end
        end
        
        cfh.children.view ="3d";
        drawlater();
//        disp(cfh.figure_id, "2) is a handle with tag = ""Robot"" ");
        // save the point of view
        point_of_view = cfh.children.rotation_angles;

        numaxh = size(cfh.children,1);
        numaxhchildren = cfh.children.children;

        if numaxh == 1 & isempty(numaxhchildren) then
//            disp(cfh.figure_id ,"3) found empty figure, just created, use it. ID = ");

            // process options
            opt = plot_options(robot,options);
            cfh.children.data_bounds = opt.workspace;
            cfh.tag = "Robot";
            h = create_new_robot(robot,opt);
            // save the handles in the passed robot object, and
            // attach it to the robot as user data.
            robot.handles = h;
//            h.links.user_data = list(robot.name, SerialLink(robot));  // robot name in user_data(1)
            h.links.user_data = list(robot.name, robot);  // robot name in user_data(1)
            // robot info in user_data(2) 
            rh = [rh; robot.handles.links];

        else
//            disp("3) reusing existing figure");
            rh = rt_findtag(robot.name);
            rh_computed = %T;
            rfigure_id = [];
            for iter=1:size(rh,1),
                rfigure_id = [rfigure_id,rh(iter).parent.parent.figure_id];
            end
            
        // Search figures with tag = "Robot" and the name of the robot.
        //           vect = [figure_ID, robot_name1, robot_name2,...]
            num_of_win = length(fh);    // used as iterator
            figure_and_names = list();
            figure_found = %F;
            handles = [];
            for i=1:num_of_win
                names = [];
                cfh = get_figure_handle(fh(i));
                if cfh.tag == "Robot" & is_handle_valid(cfh) then
                    currax = cfh.children;
                    if size(currax, "*") == 1 then
                        nobj = size(currax.children,1);
                        for iter=1:nobj,
                            if  typeof(currax.children(iter).user_data) == "list" & length(currax.children(iter).user_data) > 0 then
                                add_name = %T;
                                name = getfield(1, currax.children(iter).user_data);
                                if ~isempty(names) then
                                    for it=1:max(size(names))
                                        if names(it) == name then
                                            add_name = %F;
                                        end
                                    end
                                end

                                if add_name  then
                                    names = [names, name];
//                                    rh_computed = %T;
                                end
                            end;
                        end;
                    end;
                    figure_and_names($+1) =  [string(fh(i)),names];
                end
            end
             //----------------------------------------------
            
            if isempty(rh) | ~or(rfigure_id == cfh.figure_id) then  
                // if no robots with that name exist
                // or at least one exists but not in current figure
                // then
                // process options
 
               for iter=1:length(figure_and_names)
                   for j=2:size(figure_and_names(iter),2) // in una figura possono esserci più robot!   //j=1 is figure_ID
                    if figure_and_names(iter)(j) == robot.name & is_handle_valid(get_figure_handle(strtod(figure_and_names(iter)(1)))) then
                        cfh = scf(strtod(figure_and_names(iter)(1)));
                        figure_found = %T;
//                        rfigure_id = [rfigure_id,strtod(figure_and_names(iter)(1))];
                        j= size(figure_and_names(iter),2);
                        iter = length(figure_and_names);
                    end
                   end
               end
                if figure_found then
//                   cfh = scf(rfigure_id(1));
//                    disp(cfh.figure_id ,"4) Found figure with this robot:  cfg ID=");
                else
//                    disp("4) no robots with that name exists, hold is on/off?");
                    opt = plot_options(robot,options);
                    if opt.hold == %f then          // & length(figures) < 20
                        cfh = scf();
//                        disp(cfh.figure_id, '5) Hold off.  Create new figure ID = ');
                        drawlater();
                        cfh.children.data_bounds = opt.workspace;
                        cfh.tag = "Robot";
                    else
//                        disp('5) Hold on');
                        cfh = gcf();
                        // draw in the current figure if it contains aa robot
                        // otherwise search the last created figure cointaing a robot
                        if  cfh.tag ~= "Robot" then
                            figures = winsid();
                            i = length(figures);
                            while i > 0;
                                cfh = get_figure_handle(figures(i));
                                if cfh.tag == "Robot" & is_handle_valid(cfh) then
                                    cfh = scf(figures(i));
//                                    disp(cfh.figure_id ,'6) Add a new robot in figure ID = ');
                                    i = 0; //exit whit this figure
                                end
                                i = i-1;
                            end
                        end

                        drawlater();
                    end
                    h = create_new_robot(robot,opt);
                    robot.handles = h;
                    // robot name in user_data(1)
                    // robot info in user_data(2)
                    h.links.user_data = list(robot.name, SerialLink(robot));
                    rh = [rh; robot.handles.links];
                end
            end
        end

    end

    // now animate all robots tagged with this name

    // maintain the point of view
    cfh.children.rotation_angles = point_of_view;

    // get handle of any existing robot of same name
    if ~rh_computed then
//        disp("rh_computed is false");
        rh = rt_findtag(robot.name);
    end
    while ~opt.loop
        for p=1:np,
            for i=1:size(rh,1),
                rh_i = rh(i);
                userdata_i = rh_i.user_data;
                if is_handle_valid(cfh) then
                    execstr(rt_animate + "(userdata_i(2),tg(p,:));");
                end
            end
            if opt.delay > 0
                xpause(opt.delay);
            end
            if opt.movie then
                if ~isdir(robot.name) then
                    [status_1,msg_1] = mkdir(robot.name + "_vid");
                    if status_1 == 1 then
                        mprintf("\nplot_robot: directory ''"+ robot.name +"_vid'' created");
                    end
                end
                stringa = "xs2"+ opt.extens +"(gcf(),sprintf("""+ robot.name +"_vid/"+ robot.name +"_%03d."+ opt.extens +""",p))";
                execstr(stringa);
                sleep(100);
            end			
        end
        if ~opt.loop
            break;
        end
    end

    // save the last joint angles away in the graphical robot
    for i=1:size(rh,1),
        userdata_i = rh(i).user_data;
        rr = userdata_i(2);
        rr.q = tg($,:);
        rh(i).user_data = list(userdata_i(1), rr);
    end
    if %nargout > 0 then
        rnew = robot;
    end

    drawnow();
    cfh.immediate_drawing = "on";
    driver(old_driver);
//    disp("-------------------------------------")

endfunction
