//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [tau] = rne_mdh(robot, a1, a2, a3, a4, a5)
// Compute inverse dynamics via recursive Newton-Euler formulation
//
// Recursive Newton-Euler for modified Denavit-Hartenberg notation.  Is invoked by
// RNE(ROBOT,...).
//
// See also 
// rne
// rne_dh
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    [%nargout, %nargin] = argn(0);

    z0 = [0; 0; 1];
    grav = robot.gravity;   //default gravity from the object
    fext = zeros(6, 1);

    // Set debug to:
    //	0 no messages
    //	1 display results of forward and backward recursions
    //	2 display print R and p*
    //debug = 0;
    n = robot.n;

    if numcols(a1) == 3*n then
        Q = a1(:, 1:n);
        Qd = a1(:, n+1:2*n);
        Qdd = a1(:, 2*n+1:3*n);
        np = numrows(Q);
        if %nargin >= 3 then
            grav = a2;
        end
        if %nargin == 4 then
            fext = a3;
        end
    else
        np = numrows(a1);
        Q = a1;
        Qd = a2;
        Qdd = a3;
        if (numcols(a1) ~= n | numcols(Qd) ~= n | numcols(Qdd) ~= n | numrows(Qd) ~= np | numrows(Qdd) ~= np) then
            error("bad data");
        end
        if %nargin >= 5 then
            grav = a4;
        end
        if %nargin == 6 then
            fext = a5;
        end
    end

    tau = zeros(np,n);

    for p = 1:np,

        q = Q(p,:);
        qd = Qd(p,:);
        qdd = Qdd(p,:);

        Fm = [];
        Nm = [];
        pstarm = [];
        Pm = [];
        Rm = list();
        w = zeros(3,1);
        wd = zeros(3,1);
        v = zeros(3,1);
        vd = grav;


        //
        // init some variables, compute the link rotation matrices
        //
        for j = 1:n,
            linkj = robot.links(j);
            Tj = linkTransf(linkj,q(j))
            Rm(j) = t2r(Tj);
            if linkj.RP == "R" then
                D = linkj.d;
            else
                D = q(j);
            end
            alpha = linkj.alpha;
            Pm(:,j) = [linkj.a; -D*sin(alpha); D*cos(alpha)];   // (i-1) P i
        end


        //
        // the forward recursion
        //
        for j = 1:n,
            linkj = robot.links(j);

            R = Rm(j).';        // transpose!!
            P = Pm(:,j);
            Pc=linkj.r.';
            
            //            //    L(9)= []; //m
            //    L(10)=[]; //r
            //    L(11)=[]; //I
            //    L(12)=0;  //Jm
            //    L(13)=0;  //G
            //    L(14)=0;  //B
            // trailing underscore means new value
            //
            if linkj.RP == 'R' then                 
                // revolute axis
                w_ = R*w + z0*qd(j);
                wd_ = R*wd + cross(R*w, z0*qd(j)) + z0*qdd(j);
                //v = cross(w,P) + R*v;
                // disp(wd,"wd",w,"w",P,"P",vd,"vd:")
                vd_ = R * (cross(wd,P) + cross(w, cross(w,P)) + vd);

            else
                // prismatic axis
                w_ = R*w;
                wd_ = R*wd;
                //v = R*(z0*qd(j) + v) + cross(w,P);
                vd_ = R*(cross(wd,P) + cross(w, cross(w,P)) + vd) + 2*cross(R*w, z0*qd(j)) + z0*qdd(j);
            end
            // update variables
            w = w_;
            wd = wd_;
            vd = vd_;
            vdC = cross(wd,Pc) + cross(w, cross(w,Pc)) + vd;
            F = linkj.m*vdC;
            N = linkj.I*wd + cross(w,linkj.I*w);
            Fm = [Fm F];
            Nm = [Nm N];

        end

        //
        // the backward recursion
        //
        f = fext(1:3);      // force/moments on end of arm
        nn = fext(4:6);

        for j = n:-1:1,
            //
            // order of these statements is important, since both
            // nn and f are functions of previous f.
            //
            linkj = robot.links(j);
            if j == n then
                R = eye(3,3);
                P = [0;0;0];
            else
                R = Rm(j+1);
                P = Pm(:,j+1);      // i/P/(i+1)
            end
            Pc = linkj(10).';                           

            f_ = R*f + Fm(:,j);
            nn_ = Nm(:,j) + R*nn + cross(Pc,Fm(:,j)) + cross(P,R*f);

            f = f_;
            nn = nn_;

            if linkj.RP == "R" then
                // revolute
                tau(p,j) = nn.'*z0 + linkj(13)^2 * linkj(12)*qdd(j) + linkj(13) * friction(linkj, qd(j));
            else
                // prismatic
                tau(p,j) = f.'*z0 + linkj(13)^2 * linkj(12)*qdd(j) + linkj(13) * friction(linkj, qd(j));

            end
        end
    end

endfunction
