//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function T = rpy2tr(roll, varargin)
    // Roll-Pitch-Yaw angles to homogeneous transform
    //
    // Calling Sequence
    // T = rpy2tr(RPY, OPTIONS)
    // T = rpy2tr(ROLL, PITCH, YAW, OPTIONS)
    //
    // Description
    // T = RPY2TR(RPY, OPTIONS) is a homogeneous transformation equivalent to the 
    // specified roll, pitch,yaw angles which correspond to rotations about the 
    // X, Y, Z fixed axes respectively (RPY angles). If RPY has multiple rows they are assumed to 
    // represent a trajectory and T is a three dimensional matrix, where the last index  
    // corresponds to the rows of ANG.
    //
    // T = RPY2TR(ROLL, PITCH, YAW, OPTIONS) as above but the roll-pitch-yaw angles 
    // are passed as separate arguments. If ROLL, PITCH, YAW are column vectors 
    // they are assumed to represent a trajectory and T is a three dimensional matrix,
    // where the last index corresponds to the rows of ROLL, PITCH, YAW.
    //
    // <refsection>
    // <title>Options</title>
    // <variablelist>
    //   <varlistentry><term>'deg'     :</term>
    // 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    //   <varlistentry><term>'c'       :</term>
    // 	<listitem><para>Computes TR by means of a compiled c-function (faster)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem>
    //   <para>
    //     Input angles are handled as RPY angles
    //     and the rotation matrix is evaluated as follows
    //   </para>
    //   <para>
    //     R = Rz(roll)*Ry(pitch)*Rx(yaw).
    //   </para>
    // </listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // tr2rpy
    // rts_c_rpy2tr
    // rpy2r
    // eul2tr
    // zyz2tr
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2014 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    flag_c = %f

    out =list();
    for i = 1:(%nargin-1),
        if ~strcmp(string(varargin(i)),'c') then
            flag_c = %t;
        else
            out($+1) = varargin(i);
        end
    end
    
    if flag_c then
        try
            T = rts_c_rpy2tr(roll,out(:));
            T = matrix(T,[4 4 numrows(roll)]);
            return;
        catch
            warning(msprintf(gettext("%s: rts_c_rpy2tr not found.\n"),"rpy2tr"));
            R = rpy2r(roll, out(:));
            T = r2t(R);
            return;
        end
    end

    R = rpy2r(roll, out(:));
    T = r2t(R);

endfunction
