//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [s,sd,sdd] = tpoly(q0, qf, t, qd0, qdf)
    // Scalar polynomial trajectory
    //
    // Calling Sequence
    // [S,SD,SDD] = tpoly(S0, SF, M)
    // [S,SD,SDD] = tpoly(S0, SF, T)
    // [S,SD,SDD] = tpoly(S0, SF, N, SD0, SDF)
    // [S,SD,SDD] = tpoly(S0, SF, T, SD0, SDF)
    //
    // Description
    // <literal>[S,SD,SDD] = TPOLY(S0, SF, M)</literal> 
    // is a scalar trajectory (Mx1) that varies smoothly
    // from S0 to SF in M steps using a quintic (5th order) polynomial.
    // Velocity and acceleration can be optionally returned as SD (Mx1) 
    // and SDD (Mx1).
    //
    // <literal>[S,SD,SDD] = TPOLY(S0, SF, T)</literal> as above but specifies 
    // the trajectory in terms of the length of the time vector T (Mx1).
    //
    // <literal>[S,SD,SDD] = TPOLY(S0, SF, N, SD0, SDF)</literal> as above 
    // but specifies initial and final joint velocity for the trajectory.
    //
    // <literal>[S,SD,SDD] = TPOLY(S0, SF, T, SD0, SDF)</literal> as above
    //  but specifies initial and final joint velocity for the trajectory 
    //  and time vector T.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>In all cases if no output arguments are specified S, 
    // SD, and SDD are plotted against time.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // lspb 
    // jtraj
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    [%nargout, %nargin] = argn(0);

    t0 = t;
    if sum(length(t))==1 then//Scilab 5.5.0 -> isscalar(t)
        t = (0:t-1)';
    else
        t = t(:);
    end
    if %nargin < 4 then
        qd0 = 0;
    end
    if %nargin < 5 then
        qdf = 0;
    end

    tf = max(t);

    // solve for the polynomial coefficients using least squares
    X = [
    0           0           0         0       0   1
    tf^5        tf^4        tf^3      tf^2    tf  1
    0           0           0         0       1   0
    5*tf^4      4*tf^3      3*tf^2    2*tf    1   0
    0           0           0         2       0   0
    20*tf^3     12*tf^2     6*tf      2       0   0
    ];
    coeffs = (X \ [q0 qf qd0 qdf 0 0]')';

    // coefficients of derivatives 
    coeffs_d = coeffs(1:5) .* (5:-1:1);
    coeffs_dd = coeffs_d(1:4) .* (4:-1:1);

    // evaluate the polynomials
    p = polyval(coeffs, t);
    pd = polyval(coeffs_d, t);
    pdd = polyval(coeffs_dd, t);

    select %nargout
    case 0
        if sum(length(t))==1 // Scilab 5.5.0 -> isscalar(t0)
            // for scalar time steps, axis is labeled 1 .. M
            xt = t+1;
        else
            // for vector time steps, axis is labeled by vector M
            xt = t;
        end

        clf
        subplot(311)
        plot(xt, p); grid; ylabel('s');

        subplot(312)
        plot(xt, pd); grid; ylabel('sd');

        subplot(313)
        plot(xt, pdd); grid; ylabel('sdd');
        if sum(length(t0))<>1 then //Scilab 5.5.0 -> ~isscalar(t0)
            xlabel('time')
        else
            for c=get(gcf, 'Children');
                set(c, 'XLim', [1 t0]);
            end
        end
    case 1
        s = p;
    case 2
        s = p;
        sd = pd;
    case 3
        s = p;
        sd = pd;
        sdd = pdd;
    end
endfunction
