//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function delta = tr2delta(T0, T1)
    // Convert homogeneous transform to differential motion
    //
    // Calling Sequence
    // D = tr2delta(T0, T1)
    // D = tr2delta(T)
    //
    // Description
    // D = TR2DELTA(T0, T1) is the differential motion (6x1) corresponding to 
    // infinitessimal motion from pose T0 to T1 which are homogeneous 
    // transformations. D=(dx, dy, dz, dRx, dRy, dRz) and is an approximation
    // to the average spatial velocity multiplied by time.
    //
    // D = TR2DELTA(T) is the differential motion corresponding to the
    // infinitessimal relative pose T expressed as a homogeneous 
    // transformation.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>D is only an approximation to the motion T, and assumes
    //   that T0 ~ T1 or T ~ eye(4,4).</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // delta2tr 
    // skew
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    
    [%nargout,%nargin] = argn(0);

    if %nargin == 1
        T1 = T0;
        T0 = eye(4,4);
    end
    R0 = t2r(T0); 
    R1 = t2r(T1);
    // in world frame
    delta = [ (T1(1:3,4)-T0(1:3,4)); vex( R1*R0' - eye(3,3)) ];
    // in T0 frame
    //delta = [ R0'*(T1(1:3,4)-T0(1:3,4)); R0'*vex( R1*R0' - eye(3,3)) ];

endfunction
