//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function q = tr2q(t)
    // Convert homogeneous transform to a unit-quaternion
    //
    // Calling Sequence
    //  Q = tr2q(T)
    //
    // Description
    // Return a unit-quaternion corresponding to the rotational part of the
    // homogeneous transform T.
    //    
    // See also 
    // Quaternion
    // q2tr
    // 
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    if ishomog(t)
        t = t2r(t);
    end
    qs = sqrt(trace(t)+1)/2.0;
    kx = t(3,2) - t(2,3);   // Oz - Ay
    ky = t(1,3) - t(3,1);   // Ax - Nz
    kz = t(2,1) - t(1,2);   // Ny - Ox

    if (t(1,1) >= t(2,2)) & (t(1,1) >= t(3,3)) 
        kx1 = t(1,1) - t(2,2) - t(3,3) + 1; // Nx - Oy - Az + 1
        ky1 = t(2,1) + t(1,2);          // Ny + Ox
        kz1 = t(3,1) + t(1,3);          // Nz + Ax
        add = (kx >= 0);
    elseif (t(2,2) >= t(3,3))
        kx1 = t(2,1) + t(1,2);          // Ny + Ox
        ky1 = t(2,2) - t(1,1) - t(3,3) + 1; // Oy - Nx - Az + 1
        kz1 = t(3,2) + t(2,3);          // Oz + Ay
        add = (ky >= 0);
    else
        kx1 = t(3,1) + t(1,3);          // Nz + Ax
        ky1 = t(3,2) + t(2,3);          // Oz + Ay
        kz1 = t(3,3) - t(1,1) - t(2,2) + 1; // Az - Nx - Oy + 1
        add = (kz >= 0);
    end

    if add
        kx = kx + kx1;
        ky = ky + ky1;
        kz = kz + kz1;
    else
        kx = kx - kx1;
        ky = ky - ky1;
        kz = kz - kz1;
    end
    nm = norm([kx ky kz]);
    if nm == 0
        q = Quaternion([1 0 0 0]);
    else
        s = sqrt(1 - qs^2) / nm;
        qv = s*[kx ky kz];

        q = Quaternion([qs qv]);

    end
endfunction
