//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function euler = tr2zyz(m, varargin)
    // Convert homogeneous transform to Euler angles
    //
    // Calling Sequence
    // EUL = tr2zyz(TR, OPTIONS)
    // EUL = tr2zyz(TR, OPTIONS) 
    //
    // Parameters
    // OPTIONS = 'deg'      : compute angles in degrees (radians default)
    //
    // Description
    // EUL = TR2ZYZ(TR, OPTIONS) are the ZYZ Euler angles expressed as a row vector
    // corresponding to the rotational part of a homogeneous transform TR.
    // The 3 angles EUL=[PHI,THETA,PSI] correspond to sequential rotations about 
    // the Z, Y and Z axes respectively.
    //
    // EUL = TR2ZYZ(R, OPTIONS) are the ZYZ Euler angles expressed as a row vector
    // corresponding to the orthonormal rotation matrix R.
    //
    // If R or T represents a trajectory (has 3 dimensions), then each row of EUL
    // corresponds to a step of the trajectory.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>There is a singularity for the case where THETA=0 in which case 
    // PHI is arbitrarily set to zero and PSI is the sum (PHI+PSI).</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also  
    // zyz2tr
    // rts_c_zyz2tr
    // tr2rpy
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2014 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    opt.deg = %f;
    flag_c = %f

    out =list();
    for i = 1:(%nargin-1),
        if ~strcmp(string(varargin(i)),'c') then
            flag_c = %t;
        else
            out($+1) = varargin(i);
        end
        if ~strcmp(string(varargin(i)),'deg') then
            opt.deg = %t;
        end
    end

    if flag_c then
        try
            m = matrix(m,[4 4*size(m,3)]);
            euler = rts_c_tr2zyz(m,out(:));
            return;
        catch
            warning(msprintf(gettext("%s: rts_c_tr2zyz not found.\n"),"tr2zyz"));
            s = size(m);
            if length(s) > 2    then
                euler = zeros(s(3), 3);
                for i=1:s(3)
                    euler(i,:) = tr2zyz(m(:,:,i), out(:));
                end
                return
            end
        end
    end
    
    s = size(m);
    if length(s) > 2
        euler = zeros(s(3), 3);
        for i=1:s(3)
            euler(i,:) = tr2zyz(m(:,:,i));
        end
        return
    end

    euler = zeros(1,3);

    //	 Method as per Paul, p 69.
    //	 phi = atan2(ay, ax)
    //	 Only positive phi is returned.
    if abs(m(1,3)) < %eps & abs(m(2,3)) < %eps
        // singularity
        euler(1) = 0;
        sp = 0;
        cp = 1;
        euler(2) = atan(cp*m(1,3) + sp*m(2,3), m(3,3));
        euler(3) = atan(-sp * m(1,1) + cp * m(2,1), -sp*m(1,2) + cp*m(2,2));
    else
        euler(1) = atan(m(2,3), m(1,3));
        sp = sin(euler(1));
        cp = cos(euler(1));
        euler(2) = atan(cp*m(1,3) + sp*m(2,3), m(3,3));
        euler(3) = atan(-sp * m(1,1) + cp * m(2,1), -sp*m(1,2) + cp*m(2,2));
    end
    if opt.deg
        euler = euler * 180/%pi;
    end
endfunction
