//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function R = zyz2r(phi, varargin)
// Euler angles to rotation matrix
//
// Calling Sequence
// R = zyz2r(PHI, THETA, PSI, OPTIONS)
// R = zyz2r(EUL, OPTIONS)
//
//  Parameters
//  OPTIONS,'deg'     : Compute angles in degrees (radians default)
//
// Description
// R = ZUZ2R(PHI, THETA, PSI, OPTIONS) is an orthonornal rotation matrix 
// equivalent to the specified Euler angles.  These correspond to rotations 
// about the Z, Y, Z axes respectively. If PHI, THETA, PSI are column vectors 
// then they are assumed to represent a trajectory and R is a three dimensional 
// matrix, where the last index corresponds to rows of PHI, THETA, PSI.
//
// R = ZYZ2R(EUL, OPTIONS) as above but the Euler angles are taken from
// consecutive columns of the passed matrix EUL = [PHI THETA PSI].
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>The vectors PHI, THETA, PSI must be of the same length.</para></listitem>
// </itemizedlist>
// </refsection>
//
// See also 
// zyz2tr
// rpy2tr
// tr2zyz
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    [%nargout,%nargin] = argn(0);

    opt.deg = %f;
    if ~isempty(varargin) then
        for i = 1:(%nargin-1),
            if ~strcmp(string(varargin(i)),'deg') then
                opt.deg = ~opt.deg;
            end
        end
    end

    // unpack the arguments
    if numcols(phi) == 3
        theta = phi(:,2);
        psi = phi(:,3);
        phi = phi(:,1);
    elseif %nargin >= 3
        theta = varargin(1);
        psi = varargin(2);
    else
        error('bad arguments')
    end

    // optionally convert from degrees
    if opt.deg
        d2r = %pi/180.0;
        phi = phi * d2r;
        theta = theta * d2r;
        psi = psi * d2r;
    end

    if numrows(phi) == 1
        R = rotz(phi) * roty(theta) * rotz(psi);
    else
        R = zeros(3,3,numrows(phi));
        for i=1:numrows(phi)
            R(:,:,i) = rotz(phi(i)) * roty(theta(i)) * rotz(psi(i));
        end
    end
endfunction
